/*
 * Test dataspace operations.
 *
 * Copyright (C) 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <ipc/server.h>

#include "mapped_file_object_server.h"
#include "page_mapper.h"
#include "file_pager.h"
#include "pages.h"
#include "test_file_accessor.h"



/* A component exposing the dataspace and mapped file interfaces. */

class MappedFileServer : public FilePager, public MappedFileObject
{
public:
  explicit MappedFileServer(fileid_t fileid, PageMapper *mapper)
  : FilePager(fileid, mapper)
  {
  }

  virtual long map(unsigned long offset, l4_addr_t hot_spot, unsigned long flags, l4_snd_fpage_t *region)
  { return FilePager::map(offset, hot_spot, flags, region); }

  virtual long mmap(size_t position, size_t length, size_t *start_pos, size_t *end_pos, size_t *data_end)
  { return FilePager::mmap(position, length, start_pos, end_pos, data_end); }
};



int main(void)
{
  /* Some memory. */

  Memory mem(10);
  Pages pages(&mem);
  TestFileAccessor a(123UL);
  PageMapper m(&a, &pages);
  MappedFileServer obj(123UL, &m);

  /* Server capability. */

  l4_cap_idx_t server;

  /* Register a server associating it with the given object. */

  long err = ipc_server_bind("server", (l4_umword_t) &obj, &server);

  if (err)
  {
    printf("Could not bind server: %s\n", l4sys_errtostr(err));
    return 1;
  }

  /* Wait for messages, dispatching to the handler. */

  ipc_server_loop_for(MappedFileObject, &obj);

  return 0;
}
